/*
	------------------------------------------------------------
	ERLANG C++ Ecnapsulation: Erlang Listener Node 

		Copyright (c) 2002 by Pharos Consulting (Pty) Ltd
		Copyright (c) 2002 by Mobile Telephone Networks.
		
		
	File:		verl_listener.h

	Version:	1.00
	
	Date:		2002-08-26
	Author:		Valentin Micic, Pharos Consulting (Pty) Ltd.

	NOTES:		
	---------------------------------------------------------
	
*/
#include	<time.h>

#include	"verl.h"


// -----------------
// PRIVATE VARIABLES
// -----------------
static short			creation = 0;
static unsigned int		number	 = 0;
static unsigned int		serial	 = 0;

// ------------------------
// Construction/Destruction
// ------------------------

VERL_listener::VERL_listener(	const char* this_node_name,	const char* cookie	)
{
	bool isInitialized = false;

	if (	this_node_name	&&	
			cookie			&&
			ei_connect_init( &m_handle, this_node_name, cookie, ++creation) == 0 )
	{
		isInitialized = true;
	}
	
	if ( isInitialized == false )
	{
		memset( &m_handle, 0, sizeof( m_handle ) );
	}

	m_isStarted = false;
	m_empd_fd	= -1;
	memset( m_last_client_details.ipadr,	0, sizeof(m_last_client_details.ipadr));
	memset( m_last_client_details.nodename, 0, sizeof(m_last_client_details.nodename));

}

VERL_listener::~VERL_listener()
{
}

// -----------------
// Start/Stop Server
// -----------------

bool VERL_listener::start( int iPort )
{
	if ( m_isStarted )
		return false;

	// ---------------------------------------------------
	// First, Start Listener, and then attempt to register
	// with EPMD
	// ---------------------------------------------------

	bool bRetval = m_tcpServer.Start( iPort );
	if ( bRetval == true )
	{
		m_empd_fd = ei_publish( &m_handle, iPort );
		if ( m_empd_fd < 0 )
		{
			// -------------------------
			// Failed to publsh the port
			// -------------------------
			m_tcpServer.Stop();
			bRetval = false;
		}
	}

	m_isStarted = bRetval;
	return bRetval;

}

void VERL_listener::stop()
{
	if ( m_isStarted )
	{
		ei_unpublish( &m_handle );

#ifndef	WIN32
		close( m_empd_fd );
#else
		closesocket( m_empd_fd );
#endif

		m_tcpServer.Stop();
		m_isStarted = false;
	}

}

bool VERL_listener::accept( int & rNewFD, int msTimeout )
{
	// ------------------------------
	// Clear the last client details.
	// ------------------------------

	memset( m_last_client_details.ipadr,	0, sizeof(m_last_client_details.ipadr));
	memset( m_last_client_details.nodename, 0, sizeof(m_last_client_details.nodename));

	bool bRetval = false;
	if ( m_isStarted )
	{
		if ( m_tcpServer.Wait( msTimeout ) == true )
		{
			int & iSock = (int&)m_tcpServer.Socket();
			rNewFD = ei_accept( &m_handle, iSock, &m_last_client_details );
			if ( rNewFD == ERL_ERROR  )
			{
				rNewFD = -1;
			}
			else
			{
				bRetval = true;
			}

		}
	}

	return bRetval;
} 
